import unittest


def all_tests_suite():
    suite = unittest.TestLoader().loadTestsFromNames([
        'xattr.tests.test_xattr',
    ])
    return suite


def main():
    runner = unittest.TextTestRunner()
    suite = all_tests_suite()
    runner.run(suite)


if __name__ == '__main__':
    import os
    import sys
    sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))
    main()


infile = '/usr/include/sys/xattr.h'
out = file('Lib/xattr/constants.py', 'w')
for line in file(infile):
    line = line.rstrip()
    if line.startswith('/*') and line.endswith('*/'):
        print >>out,  ''
        print >>out,  '# ' + line[2:-2].strip()
    elif line.startswith('#define'):
        if lastblank:
            print >>out,  ''
        chunks = line.split(None, 3)
        if len(chunks) == 3:
            print >>out,  '%s = %s' % (chunks[1], chunks[2])
        elif len(chunks) == 4:
            comment = chunks[3].replace('/*', '').replace('*/', '').strip()
            print >>out,  '%s = %s # %s' % (chunks[1], chunks[2], comment)
    if not line:
        lastblank = True
    else:
        lastblank = False




# Options for pathname based xattr calls
XATTR_NOFOLLOW = 0x0001 # Don't follow symbolic links

# Options for setxattr calls
XATTR_CREATE = 0x0002 # set the value, fail if attr already exists
XATTR_REPLACE = 0x0004 # set the value, fail if attr does not exist

# Set this to bypass authorization checking (eg. if doing auth-related work)
XATTR_NOSECURITY = 0x0008

XATTR_MAXNAMELEN = 127

XATTR_FINDERINFO_NAME = "com.apple.FinderInfo"

XATTR_RESOURCEFORK_NAME = "com.apple.ResourceFork"



import os
from unittest import TestCase
from tempfile import mkdtemp, NamedTemporaryFile

import xattr

class TestFile(TestCase):
    def setUp(self):
        self.tempfile = NamedTemporaryFile()
        self.tempfilename = self.tempfile.name

    def tearDown(self):
        self.tempfile.close()

    def testAttr(self):
        x = xattr.xattr(self.tempfile)
        self.assertEqual(x.keys(), [])
        self.assertEqual(dict(x), {})

        x['user.sopal'] = 'foo'
        x['user.sop.foo'] = 'bar'
        del x

        x = xattr.xattr(self.tempfile)
        self.assertTrue('user.sopal' in x)
        self.assertEqual(x['user.sopal'], 'foo')
        self.assertTrue('user.sop.foo' in x)
        self.assertEqual(x['user.sop.foo'], 'bar')

        del x['user.sop.foo']
        del x

        x = xattr.xattr(self.tempfile)
        self.assertTrue('user.sop.foo' not in x)

    def testSymlinkAttrs(self):
        symlinkPath = self.tempfilename + '.link'
        os.symlink(self.tempfilename, symlinkPath)
        try:
            symlink = xattr.xattr(symlinkPath, options=xattr.XATTR_NOFOLLOW)
            realfile = xattr.xattr(self.tempfilename)
            symlink['user.islink'] = 'true'
            self.assertEqual(dict(realfile), {})
            self.assertEqual(symlink['user.islink'], 'true')
        finally:
            os.remove(symlinkPath)

class TestDir(TestFile):
    def setUp(self):
        self.tempfile = mkdtemp()
        self.tempfilename = self.tempfile

    def tearDown(self):
        os.rmdir(self.tempfile)



#!/usr/bin/env python

##
# Copyright (c) 2007 Apple Inc.
#
# This is the MIT license.  This software may also be distributed under the
# same terms as Python (the PSF license).
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.
##

import sys
import os
import getopt
import xattr
import zlib

def usage(e=None):
    if e:
        print e
        print ""

    name = os.path.basename(sys.argv[0])
    print "usage: %s [-lz] file [file ...]" % (name,)
    print "       %s -p [-lz] attr_name file [file ...]" % (name,)
    print "       %s -w [-z] attr_name attr_value file [file ...]" % (name,)
    print "       %s -d attr_name file [file ...]" % (name,)
    print ""
    print "The first form lists the names of all xattrs on the given file(s)."
    print "The second form (-p) prints the value of the xattr attr_name."
    print "The third form (-w) sets the value of the xattr attr_name to attr_value."
    print "The fourth form (-d) deletes the xattr attr_name."
    print ""
    print "options:"
    print "  -h: print this help"
    print "  -l: print long format (attr_name: attr_value)"
    print "  -z: compress or decompress (if compressed) attribute value in zip format"

    if e:
        sys.exit(64)
    else:
        sys.exit(0)

class NullsInString(Exception):
    """Nulls in string."""

_FILTER=''.join([(len(repr(chr(x)))==3) and chr(x) or '.' for x in range(256)])

def _dump(src, length=16):
    result=[]
    for i in xrange(0, len(src), length):
        s = src[i:i+length]
        hexa = ' '.join(["%02X"%ord(x) for x in s])
        printable = s.translate(_FILTER)
        result.append("%04X   %-*s   %s\n" % (i, length*3, hexa, printable))
    return ''.join(result)

def main():
    try:
        (optargs, args) = getopt.getopt(sys.argv[1:], "hlpwdz", ["help"])
    except getopt.GetoptError, e:
        usage(e)

    attr_name   = None
    long_format = False
    read        = False
    write       = False
    delete      = False
    compress    = lambda x: x
    decompress  = compress
    status      = 0

    for opt, arg in optargs:
        if opt in ("-h", "--help"):
            usage()
        elif opt == "-l":
            long_format = True
        elif opt == "-p":
            read = True
            if write or delete:
                usage("-p not allowed with -w or -d")
        elif opt == "-w":
            write = True
            if read or delete:
                usage("-w not allowed with -p or -d")
        elif opt == "-d":
            delete = True
            if read or write:
                usage("-d not allowed with -p or -w")
        elif opt == "-z":
            compress   = zlib.compress
            decompress = zlib.decompress

    if write or delete:
        if long_format:
            usage("-l not allowed with -w or -p")

    if read or write or delete:
        if not args:
            usage("No attr_name")
        attr_name = args.pop(0)

    if write:
        if not args:
            usage("No attr_value")
        attr_value = args.pop(0)

    if len(args) > 1:
        multiple_files = True
    else:
        multiple_files = False

    for filename in args:
        def onError(e):
            if not os.path.exists(filename):
                sys.stderr.write("No such file: %s\n" % (filename,))
            else:
                sys.stderr.write(str(e) + "\n")
            status = 1

        try:
            attrs = xattr.xattr(filename)
        except (IOError, OSError), e:
            onError(e)
            continue

        if write:
            try:
                attrs[attr_name] = compress(attr_value)
            except (IOError, OSError), e:
                onError(e)
                continue

        elif delete:
            try:
                del attrs[attr_name]
            except (IOError, OSError), e:
                onError(e)
                continue
            except KeyError:
                onError("No such xattr: %s" % (attr_name,))
                continue

        else:
            try:
                if read:
                    attr_names = (attr_name,)
                else:
                    attr_names = attrs.keys()
            except (IOError, OSError), e:
                onError(e)
                continue

            if multiple_files:
                file_prefix = "%s: " % (filename,)
            else:
                file_prefix = ""

            for attr_name in attr_names:
                try:
                    try:
                        attr_value = decompress(attrs[attr_name])
                    except zlib.error:
                        attr_value = attrs[attr_name]
                except KeyError:
                    onError("%sNo such xattr: %s" % (file_prefix, attr_name))
                    continue

                if long_format:
                    try:
                        if attr_value.find('\0') >= 0:
                            raise NullsInString;
                        print "".join((file_prefix, "%s: " % (attr_name,), attr_value))
                    except (UnicodeDecodeError, NullsInString):
                        print "".join((file_prefix, "%s:" % (attr_name,)))
                        print _dump(attr_value)
                else:
                    if read:
                        print "".join((file_prefix, attr_value))
                    else:
                        print "".join((file_prefix, attr_name))

    sys.exit(status)

if __name__ == "__main__":
    main()



"""
Extended attributes extend the basic attributes of files and directories
in the file system.  They are stored as name:data pairs associated with
file system objects (files, directories, symlinks, etc).

The xattr type wraps a path or file descriptor with a dict-like interface
that exposes these extended attributes.
"""

__version__ = '0.6.4'
from constants import XATTR_NOFOLLOW, XATTR_CREATE, XATTR_REPLACE, \
    XATTR_NOSECURITY, XATTR_MAXNAMELEN, XATTR_FINDERINFO_NAME, \
    XATTR_RESOURCEFORK_NAME

import _xattr

def _pyflakes_api():
    # trick pyflakes into thinking these are used.
    return [
        XATTR_NOFOLLOW, XATTR_CREATE, XATTR_REPLACE,
        XATTR_NOSECURITY, XATTR_MAXNAMELEN, XATTR_FINDERINFO_NAME,
        XATTR_RESOURCEFORK_NAME,
    ]


def _boundfunc(func, first):
    def _func(*args):
        return func(first, *args)
    return _func

class xattr(object):
    """
    A wrapper for paths or file descriptors to access
    their extended attributes with a dict-like interface
    """

    def __init__(self, obj, options=0):
        """
        obj should be a path, a file descriptor, or an
        object that implements fileno() and returns a file
        descriptor.

        options should be 0 or XATTR_NOFOLLOW.  If set, it will
        be OR'ed with the options passed to getxattr, setxattr, etc.
        """
        self.obj = obj
        self.options = options
        self.flavor = None
        fileno = getattr(obj, 'fileno', None)
        if fileno is not None:
            obj = fileno()
        if isinstance(obj, int):
            self.flavor = 'fd'
            self._bind_any('f%sxattr', obj, options)
        else:
            self.flavor = 'file'
            self._bind_any('%sxattr', obj, options)

    def __repr__(self):
        if self.flavor:
            return '<%s %s=%r>' % (type(self).__name__, self.flavor, self.obj)
        else:
            return object.__repr__(self)

    def _bind_any(self, fmt, obj, options):
        options = self.options
        for method in ("get", "set", "remove", "list"):
            name = '_' + method
            func = getattr(_xattr, fmt % (method,))
            meth = _boundfunc(func, obj)
            try:
                meth.__name__ = name
            except TypeError:
                pass
            setattr(self, name, meth)

    def get(self, name, options=0):
        """
        Retrieve the extended attribute ``name`` as a ``str``.
        Raises ``IOError`` on failure.

        See x-man-page://2/getxattr for options and possible errors.
        """
        return self._get(name, 0, 0, options | self.options)

    def set(self, name, value, options=0):
        """
        Set the extended attribute ``name`` to ``value``
        Raises ``IOError`` on failure.

        See x-man-page://2/setxattr for options and possible errors.
        """
        self._set(name, value, 0, options | self.options)

    def remove(self, name, options=0):
        """
        Remove the extended attribute ``name``
        Raises ``IOError`` on failure.

        See x-man-page://2/removexattr for options and possible errors.
        """
        self._remove(name, options | self.options)

    def list(self, options=0):
        """
        Retrieves the extended attributes currently set as a list
        of unicode strings.  Raises ``IOError`` on failure.

        See x-man-page://2/listxattr for options and possible errors.
        """
        res = self._list(options | self.options).split('\x00')
        res.pop()
        return [unicode(s, 'utf-8') for s in res]

    # dict-like methods

    def __len__(self):
        return len(self.list())

    def __delitem__(self, item):
        try:
            self.remove(item)
        except IOError:
            raise KeyError(item)

    def __setitem__(self, item, value):
        self.set(item, value)

    def __getitem__(self, item):
        try:
            return self.get(item)
        except IOError:
            raise KeyError(item)

    def iterkeys(self):
        return iter(self.list())

    __iter__ = iterkeys

    def has_key(self, item):
        try:
            self.get(item)
        except IOError:
            return False
        else:
            return True

    __contains__ = has_key

    def clear(self):
        for k in self.keys():
            del self[k]

    def update(self, seq):
        if not hasattr(seq, 'iteritems'):
            seq = dict(seq)
        for k, v in seq.iteritems():
            self[k] = v

    def copy(self):
        return dict(self.iteritems())

    def setdefault(self, k, d=''):
        try:
            d = self.get(k)
        except IOError:
            self[k] = d
        return d

    def keys(self):
        return self.list()

    def itervalues(self):
        for k, v in self.iteritems():
            yield v

    def values(self):
        return list(self.itervalues())

    def iteritems(self):
        for k in self.list():
            yield k, self.get(k)

    def items(self):
        return list(self.iteritems())


def listxattr(f, symlink=False):
    __doc__ = xattr.list.__doc__
    return tuple(xattr(f).list(options=symlink and XATTR_NOFOLLOW or 0))

def getxattr(f, attr, symlink=False):
    __doc__ = xattr.get.__doc__
    return xattr(f).get(attr, options=symlink and XATTR_NOFOLLOW or 0)

def setxattr(f, attr, value, options=0, symlink=False):
    __doc__ = xattr.set.__doc__
    if symlink:
        options |= XATTR_NOFOLLOW
    return xattr(f).set(attr, value, options=options)

def removexattr(f, attr, symlink=False):
    __doc__ = xattr.remove.__doc__
    options = symlink and XATTR_NOFOLLOW or 0
    return xattr(f).remove(attr, options=options)



#!/usr/bin/env python

import ez_setup
ez_setup.use_setuptools()

from setuptools import setup, Extension

VERSION = '0.6.4'
DESCRIPTION = "Python wrapper for extended filesystem attributes"
LONG_DESCRIPTION = """
Extended attributes extend the basic attributes of files and directories
in the file system.  They are stored as name:data pairs associated with
file system objects (files, directories, symlinks, etc).

Extended attributes are currently only available on Darwin 8.0+ (Mac OS X 10.4)
and Linux 2.6+. Experimental support is included for Solaris and FreeBSD.
"""

CLASSIFIERS = filter(None, map(str.strip,
"""
Environment :: Console
Intended Audience :: Developers
License :: OSI Approved :: MIT License
Natural Language :: English
Operating System :: MacOS :: MacOS X
Operating System :: POSIX :: Linux
Operating System :: POSIX :: BSD :: FreeBSD
Programming Language :: Python
Topic :: Software Development :: Libraries :: Python Modules
""".splitlines()))

setup(
    name="xattr",
    version=VERSION,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    classifiers=CLASSIFIERS,
    author="Bob Ippolito",
    author_email="bob@redivi.com",
    url="http://github.com/xattr/xattr",
    license="MIT License",
    packages=['xattr'],
    platforms=['MacOS X', 'Linux', 'FreeBSD', 'Solaris'],
    ext_modules=[
        Extension("xattr._xattr", ["xattr/_xattr.c"]),
    ],
    entry_points={
        'console_scripts': [
            "xattr = xattr.tool:main",
        ],
    },
    test_suite="xattr.tests.all_tests_suite",
    zip_safe=False,
)
